function [peakVals,mainComputationTime,nVals,aVals,p]=paramSweepParfeval(nVals,hVals,aVals,L,showTruss,hTopAxes) 
% Parameter sweep study to investigate deflection in a 2D cantilevered truss
%
% Assumes fixed material properties and fixed (configurable) length of truss
%
% Inputs
%    nVals, vector of number of horizontal segments (recommend values in range [2 20])
%    hVals, vector of height of truss (recommended values in range [0.1 1])
%    aVals, vector of cross section area for each rod (recommended values in range [ 0.1 1])
%    L, horizontal length of the truss (suggested value 1)
%
% Examples:
%   [~,mainComputationTime]=paramSweepParallel(2:1:18,0.1:.03:.7,[0.25],1)
%   [~,mainComputationTime]=paramSweepParallel(2:2:20,0.1:.05:.6,[0.25 0.5],1)
%   [~,mainComputationTime]=paramSweepParallel(2:1:18,0.1:.01:.7,[0.25],1)
%
%
% Copyright 2014-2016 The MathWorks, Inc.

if nargin<5
    showTruss=0;
end

if nargin<6
    hTopAxes=[];
end


%% Set up problem to provide a single index to loop on
[nGrid, hGrid, aGrid] = meshgrid(nVals, hVals, aVals);
peakVals = nan(size(nGrid));

if ~isempty(hTopAxes)
    visualizeParamSweep(nVals, hVals, aVals, peakVals, hTopAxes);
end

%% Parameter Sweep with parfeval
t0 = tic;

% Note: with many iterations, overhead will be large, so we submit groups of iterates
p = gcp;
N = min([3*p.NumWorkers, numel(nGrid)]);   % similar to what parfor does (recommended for non-trivial cases)
%N = numel(nGrid);     % Basically turns off the grouper (an over-ride)

% Asynchronous submission
for ii = 1:N
    idx=ii:N:numel(nGrid);  % take every Nth sample, so we can see results fill in
    
    % see the Grouper function below for the function calls that match the parfor version
    % Note that parfeval will submit, but not waitfor results
    f(idx)=parfeval(p, @Grouper, 6, nGrid(idx), hGrid(idx), aGrid(idx), L, idx); %#ok<AGROW>
end

% Can do other work while I wait ( Asyncrhonous)

% Synchronous retrieval
for ii=1:N
    % fetchNext will block additional commands until next result is retrieved
    [~, thisResult, Y, bars, groundDofs, actualToReduced, thisIdx] = fetchNext(f);
    
    peakVals(thisIdx) = thisResult;
    
    if ~isempty(hTopAxes)
        hTopAxes.Children.ZData(thisIdx)=log10(abs(peakVals(thisIdx)));
        drawnow
    end
    
    % Plot trusses
    if showTruss
        plotTruss(Y,bars,L,nGrid(thisIdx),hGrid(thisIdx),groundDofs,actualToReduced);
    end
    
end

mainComputationTime = toc(t0);

if ~isempty(hTopAxes)
    visualizeParamSweep(nVals, hVals, aVals, peakVals, hTopAxes);
end

end

function [peakVals, Y, bars, groundDofs, actualToReduced, idx]=Grouper(nVals,hVals,aVals,L,idx)
% Group sets of calculations to reduce effects of overhead
% Note in this version, we only output teh last set of data for plotTruss

peakVals = zeros(length(nVals),1);
for ii = 1:length(hVals)
    [isModelAssumptionValid,Y,bars,groundDofs,actualToReduced] = trussCantilever(nVals(ii),hVals(ii),aVals(ii),L); 
    if isModelAssumptionValid
        % Determine peak Y deflection at any node
        % columns are all x,y pairs followed by all xdot,ydot pairs
        peakVals(ii) = max(max(abs(Y(:,2:2:end/2))));
    else
        fprintf('Linear model assumption not valid for N = %d and A = %e\n',nGrid(ii),hGrid(ii),aGrid(ii));
        peakVals(ii) = nan;
    end
end

end